/*
// A simple C-style library for polynomials
//
*/

#include "poly.h"

int
main() {

  Poly p, q, r, pq ;

  // create a polynomial
  PolyCreate( p,  2 ) ;
  PolyCreate( q,  2 ) ;
  PolyCreate( r,  2 ) ;
  PolyCreate( pq, 0 ) ;

  PolySet(p, 1,  2) ;
  PolySet(p, -1, 1) ;
  PolySet(q, 2,  0) ;
  PolySet(q, 1,  1) ;

  printf("\npolynomial multiplication\n") ;
  printf("p   = ") ; PolyPrint( p, stdout ) ;
  printf("q   = ") ; PolyPrint( q, stdout ) ;
  
  PolyMUL( p, q, r ) ;
  printf("p*q = ") ; PolyPrint( r, stdout ) ;

  printf("\npolynomial derivative\n") ;
  printf("p   = ") ; PolyPrint( p,  stdout ) ;
  PolyDiff( p, pq ) ;
  printf("Dp  = ") ; PolyPrint( pq,  stdout ) ;

  printf("\npolynomial integral\n") ;
  printf("p   = ") ; PolyPrint( p,  stdout ) ;
  PolyInt( p, pq ) ;
  printf("Ip  = ") ; PolyPrint( pq,  stdout ) ;

  printf("\npolynomial evaluation\n") ;
  printf("p   = ") ; PolyPrint( p,  stdout ) ;
  valueType v = 10 ;
  printf("p(%f)=%f\n",v, PolyEval(p,v) ) ;
  v = -10 ;
  printf("p(%f)=%f\n",v, PolyEval(p,v) ) ;

  printf("\npolynomial division\n") ;
  valueType cfp[] = {1,2,3,4,5,6,7,8} ;
  valueType cfq[] = {1,2,3,4,5} ;
  PolySetValues( p, cfp, 8 ) ;
  PolySetValues( q, cfq, 5 ) ;
  
  PolyDIV( p, q, pq, r ) ;
  printf("p   = ") ; PolyPrint( p,  stdout ) ;
  printf("q   = ") ; PolyPrint( q,  stdout ) ;
  printf("p/q = ") ; PolyPrint( pq, stdout ) ;
  printf("r   = ") ; PolyPrint( r,  stdout ) ;

  printf("\npolynomial MCD\n") ;
  indexType nPoly ;
  Poly polys[100] ;
  PolyMCD( p, q, nPoly, polys ) ;
  for ( indexType i = 0 ; i < nPoly ; ++i )
    { printf("p%d = ",i) ; PolyPrint( polys[i],  stdout ) ; }

  printf("\nSturm sequence\n") ;
  PolySturm( p, nPoly, polys ) ;
  for ( indexType i = 0 ; i < nPoly ; ++i )
    { printf("p%d = ",i) ; PolyPrint( polys[i],  stdout ) ; }

  for ( valueType x = -1 ; x < 1 ; x += 0.1 ) {
    printf( "Sturm changes x=%f %d : ", x, PolySturmChanges( nPoly, polys, x ) ) ;
    PolyPrintSturmChanges( stdout, nPoly, polys, x ) ;
  }

  return 0 ;
}