/*
// A simple C++-style interface for polynomials
//
*/

#include "poly.h"
#include <iostream>
#include <iomanip>

using namespace std ;

int
main() {
  
  valueType cfs[]  = {1,2,3,4,5} ;
  indexType numCfs = sizeof(cfs)/sizeof(cfs[0]) ;

  Poly p(10), q(10), r(10), pq(cfs,numCfs) ;

  p[2] = 1  ;
  p[1] = -1 ;
  p[0] = 2  ;
  q[1] = 1  ;
  r    = p*q ;
  cout << "\npolynomial multiplication\n"
       << "p   = " << p << '\n'
       << "q   = " << q << '\n'
       << "p*q = " << r << '\n' ;

  pq = diff(p) ;
  cout << "\npolynomial derivative\n"
       << "p   = " << p  << '\n'
       << "Dp  = " << pq << '\n' ;

  //cout << "\npolynomial integral\n") ;
  //printf("p   = ") ; PolyPrint( p,  stdout ) ;
  //PolyInt( p, pq ) ;
  // printf("Ip  = ") ; PolyPrint( pq,  stdout ) ;

  cout << "\npolynomial evaluation\n"
       << "p      = " << p      << '\n'
       << "p(10)  = " << p(10)  << '\n'
       << "p(-10) = " << p(-10) << '\n' ;

  valueType cfp[] = {1,2,3,4,5,6,7,8} ;
  valueType cfq[] = {1,2,3,4,5} ;
  p . setup( cfp, 8 ) ;
  q . setup( cfq, 5 ) ;

  cout << "\npolynomial division\n" ;
  div( p, q, pq, r ) ;
  cout << "p   = " << p  << '\n'
       << "q   = " << q  << '\n'
       << "p/q = " << pq << '\n'
       << "r   = " << r  << '\n' ;

  cout << "\npolynomial MCD\n" ;
  indexType nPoly ;
  Poly polys[100] ;
  mcd( p, q, nPoly, polys ) ;
  for ( indexType i = 0 ; i < nPoly ; ++i )
    cout << "p" << i << " = " << polys[i] << '\n' ;

  cout << "\nSturm sequence\n" ;
  //valueType cfpp[] = {-6,11,-6,1} ;
  //p . setup( cfpp, 4 ) ;
  Sturm( p, nPoly, polys ) ;
  for ( indexType i = 0 ; i < nPoly ; ++i )
    cout << "p" << i << " = " << polys[i] << '\n' ;

  for ( valueType x = -1 ; x < 1 ; x += 0.5 ) {
    cout << "Sturm changes x=" << setw(10) << x << " " 
         << SturmChanges( nPoly, polys, 1E-10, x ) ;
    SturmChanges( cout, nPoly, polys, 1E-10, x ) ;
  }

  return 0 ;
}