/*
// A simple C++-style interface for polynomials.
// It implements some operations on polynomials.
//
*/

#include <iostream>

using namespace std ;

// parametrize type: valueType is the type for floating point values
typedef double valueType ; // in STL value_type

// parametrize type: indexType is the type for integer values
typedef int indexType ;

/*
//  The type struct Poly, contain the needed information to store a polynomial
*/

class Poly {

public:

  // constructors
  Poly(Poly const &) ;
  
  explicit Poly() ;
  explicit Poly(indexType order) ;
  explicit Poly(valueType coeffs[], indexType numCoeffs) ;

  // destructor
  ~Poly() ;

  Poly const & operator =  ( Poly const & p ) ;
  Poly const & operator =  ( valueType s ) ;
  valueType    operator () ( valueType x ) const ;

  Poly const & operator += ( Poly const & p ) ;
  Poly const & operator -= ( Poly const & p ) ;

  valueType const & operator [] (indexType i) const
  { return coeffs[i] ; }

  valueType & operator [] (indexType i)
  { return coeffs[i] ; }

  void      setup (valueType coeffs[], indexType numCoeffs) ;
  void      resize( indexType newOrder ) ;
  void      drop  ( valueType epsi ) ;
  void      print( ostream & s ) const ;
  indexType getOrder() const ;
  valueType leading() const ;

private:
  valueType * coeffs ;
  indexType order ;
};

// define the operator (ostream << Poly)
inline
ostream &
operator << ( ostream & s, Poly const & p )
{ p.print(s) ; return s ; }

// negate a polynomial
Poly operator - ( Poly const & a ) ;

// algebra for polynomial
Poly operator + ( Poly const & a, Poly const & b ) ;
Poly operator - ( Poly const & a, Poly const & b ) ;
Poly operator * ( Poly const & a, Poly const & b ) ;

// mixed scalar/polynomial
Poly operator + ( valueType a, Poly const & b ) ;
Poly operator - ( valueType a, Poly const & b ) ;
Poly operator * ( valueType a, Poly const & b ) ;
Poly operator - ( Poly const & a, valueType b ) ;
Poly operator + ( Poly const & a, valueType b ) ;
Poly operator * ( Poly const & a, valueType b ) ;
Poly operator / ( Poly const & a, valueType b ) ;

// divide two polynomial a/b return the quotient and the rest
// a = p * b + r
void div( Poly const & a, Poly const & b, Poly & p, Poly & r ) ;
void mcd( Poly const & p, Poly const & q, indexType & nPoly, Poly polys[] ) ;
// compute polynomial derivative
Poly diff( Poly const & p ) ;
void Sturm( Poly const & p, indexType & nPoly, Poly polys[] ) ;
indexType SturmChanges( indexType nPoly, Poly const polys[], valueType epsi, valueType x ) ;
void      SturmChanges( ostream & s, indexType nPoly, Poly const polys[], valueType epsi, valueType x ) ;

// end of file poly.h
